import { Client, isFullPage } from "@notionhq/client"
import {
  PageObjectResponse,
  QueryDatabaseResponse,
} from "@notionhq/client/build/src/api-endpoints"
import { Rate } from "../model"

export class NotionProvider {
  #NOTION_KEY: string
  #DATABASE_ID: string

  public constructor(notionKey: string, databaseId: string) {
    this.#NOTION_KEY = notionKey
    this.#DATABASE_ID = databaseId
  }

  public async queryRates(currencies: Array<string>): Promise<Rate> {
    const client = new Client({
      auth: this.#NOTION_KEY,
      fetch: fetch.bind(globalThis),
    })

    let query: QueryDatabaseResponse
    let startCursor: string | null = null
    let pages: Array<PageObjectResponse> = []
    do {
      query = await client.databases.query({
        database_id: this.#DATABASE_ID,
        page_size: 100,
        start_cursor: startCursor ?? undefined,
        filter: {
          or: currencies.map(currency => ({
            type: "title",
            property: "Code",
            title: {
              equals: currency,
            },
          })),
        },
      })

      const queryResults = query.results.filter(page => isFullPage(page))
      startCursor = query.next_cursor

      pages = [...pages, ...queryResults]
    } while (query.next_cursor !== null)

    let ratesMap: Rate = {}
    for (const page of pages) {
      let currencyCodeProp = page.properties["Code"]
      let rateProp = page.properties["Rate"]
      if (currencyCodeProp!.type !== "title") {
        continue
      }
      if (rateProp!.type !== "number") {
        continue
      }
      ratesMap[currencyCodeProp.title[0]!.plain_text] = rateProp.number!
    }

    return ratesMap
  }
}
